# OVERVIEW

[Earthworm](http://gitlab.com/seismic-software/earthworm/) is a free, open-source, modular, and extensible waveform and automatic earthquake processing software package written primarily in the C language.  Originally developed by the United States Geological Survey ([USGS](http://www.usgs.gov)), Earthworm has been an open-source project since 1993, with contributors from all over the world.

In an Earthworm system, processing is performed by separate programs, called *modules*.  Each module performs a specific task and communicates with the other Earthworm modules by writing typed, ordered *message* packets into shared memory regions, called *rings*.

In a typical Earthworm system, a device-specific module obtains digitized samples (*waveform data*) of the signal measured by one or more sensors attached to a device (*channels*), such as an A/D board or a seismograph.  The module converts the data into a sequence of standard Earthworm waveform data message packets, called *TRACEBUFs*.  A TRACEBUF identifies the source of the data it contains using a unique signature for each channel, called a *SCNL*.  The waveform data segment it contains is identified by the *data type* of each sample, the *number of samples*, the *UTC time* of the first sample, and the *sample rate*.

The module writes each TRACEBUF, in order, into a shared memory ring for further processing by other modules.  Various processing modules then operate in conjunction on the messages in one or more rings as they consume data from their upstream providers and produce processed data for their downstream consumers.

SCNL is an acronym for *Station ID*, *Channel ID*, *Network Code*, and *Location ID*.  Earthworm follows the SCNL naming convention specified in the [Standard for the Exchange of Earthquake Data (SEED) Reference Manual](http://www.fdsn.org/pdf/SEEDManual_V2.4.pdf):

	Station ID      1 to 5 [UN] characters  
	Channel ID      3 [UN] characters  
	Network Code    1 or 2 [ULN] characters  
	Location ID     1 or 2 [UN] characters, or the reserved characters "--", if omitted  

	[UN]  is the set of upper case and numeric ASCII characters  
	[ULN] is the set of upper case, lower case, and numeric ASCII characters

Network codes are globally unique, and are assigned by the International Federation of Digital Seismograph Networks ([FDSN](http://www.fdsn.org/networks/)).  Each network operator assigns the remaining identifiers for use within their own network.

The Station ID refers to the particular site (*station*) of one or more a sensors within a network, such as a 3-component accelerometer package oriented Up-Down (*Z*), N-S (*N*), and E-W (*E*).

The Channel ID encodes the band (based on the sample rate), sensor type, and sensor orientation.  Channel IDs are assigned following *Appendix A: Channel Naming* in the [SEED Reference Manual](http://www.fdsn.org/pdf/SEEDManual_V2.4.pdf).

Location IDs are optional.  They are used as necessary to differentiate sensor packages deployed at the same site, but at different locations.  For example, sensors placed at different depths in a borehole array.  Or, sensors placed throughout a building to measure the structural response to strong ground shaking or high wind loads.

> *NOTE: The data source identifier specification defined by the FDSN (<http://docs.fdsn.org/projects/source-identifiers/en/v1.0/>) provides for significantly longer network codes, station IDs, and location IDs than does SEED format.  At present, Earthworm is not compatible with the longer identifiers permitted by the FDSN; data interchange with some FDSN networks may not be possible.*

Example Earthworm SCNLs for a hypothetical USGS Earthquake Early Warning (*EEW*) station at Zzyzx, California, with a surface sensor package (no location ID) and a downhole string of sensors at 5 m depth (location D1), 10 m depth (location D2), etc.:

	  S    C  N  L    Network   Station   Location  Channel (encoded: band, sensor, orientation)
	----- --- -- --   -------- --------- ---------- ---------------------------------------------
	ZZYZX HNZ EW --   USGS EEW Zzyzx, CA  surface   H:200 sps, N:accelerometer,         Z:Up-Down
	      HNN    --                       surface   H:200 sps, N:accelerometer,         N:N-S
	      HNE    --                       surface   H:200 sps, N:accelerometer,         E:E-W
	      HHZ    D1                       5 m depth H:100 sps, H:broadband seismometer, Z:Up-Down
	      HHN    D1                       5 m depth H:100 sps, H:broadband seismometer, N:N-S
	      HHE    D1                       5 m depth H:100 sps, H:broadband seismometer, E:E-W
	      HHZ    D2                      10 m depth H:100 sps, H:broadband seismometer, Z:Up-Down
	      HHN    D2                      10 m depth H:100 sps, H:broadband seismometer, N:N-S
	      HHE    D2                      10 m depth H:100 sps, H:broadband seismometer, E:E-W

Earthworm includes nearly 200 modules, categorized by purpose, such as seismic processing, archiving, system control, etc.  For example, if you have data from at least four stations, there are standard Earthworm seismic processing modules to associate phase arrivals and compute earthquake locations.  Existing modules can be modified, or new modules written, to provide additional features.

An Earthworm system runs on a single computer.  However, Earthworm provides modules for standard protocols used by regional seismic networks to share waveforms, picks, and other information, whether by exchanging data between export/import modules running on multiple Earthworm instances, or through the use of public information services, for example, by hosting web services or issuing notifications via emails, SMS texts, pagers, etc.

The USGS funds Instrumental Software Technologies, Inc. ([ISTI](http://www.isti.com)), for maintenance, development, and distribution of new Earthworm releases.  The University of Memphis and ISTI host the Earthworm documentation and distribution web sites.

# DOCUMENTATION

The Earthworm documentation can be found within this distribution at `doc/WEB_DOC/index.html`.  The same documentation can be found online at <http://folkworm.ceri.memphis.edu/ew-doc/> or <http://earthwormcentral.org/>.

# DOWNLOAD EARTHWORM

Earthworm releases are available at <http://folkworm.ceri.memphis.edu/ew-dist/> and <http://earthwormcentral.com/distribution/>.  Pre-compiled binary distributions are available for several operating systems and hardware architectures.  If a suitable binary distribution is not available for your computer, you will have to build the binaries from an Earthworm source distribution, found at the same web sites.  (See COMPILING below.)

Alternately, you can download a snapshot of the current Earthworm source and build your own Earthworm from the Earthworm GitLab repository, http://gitlab.com/seismic-software/earthworm/.  This is only recommended for experienced Earthworm programmers, since the current Earthworm source may be unstable.

# GETTING STARTED

## Minimum Earthworm Setup

- Create the parameter and log file directories
- Find the existing Installation ID for your institution or acquire a new one
- Set the environment variables in the run script for your configuration
- Add the modules you want to use to the `startstop` configuration file
- Make sure all the modules you use have a unique Module ID in the `earthworm.d` configuration file
- Add the modules you want to monitor to the `statmgr` configuration file
- Configure each module's `.d` configuration file for `startstop`
- For modules being monitoried by `statmgr`, configure each module's `.desc` configuration file

## The Earthworm Directory Structure

		EW_HOME/
		├── EW_VERSION/
		│   └──bin/         Earthworm executables are here
		└── run_working/
		    ├── data/       EW_DATA_DIR
		    │   ├── mseed/  E.g., miniSEED event files                                                                                    
		    │   └── heli/   E.g., pseuso-helicorder web pages
		    ├── log/        EW_LOG
		    └── params/     EW_PARAMS

Choose a suitable location for the Earthworm top-level directory on your system.  The `EW_HOME` environment variable stores this path name.  Unpack the Earthworm distribution into this directory.  Every time there is a new Earthworm release, the name of the directory containing the release will change, such as `earthworm_7.9`, `earthworm_7.10`, etc.  The `EW_VERSION` environment variable stores the directory name of the release you are using.  The combination of `EW_HOME` and `EW_VERSION` determines the location of the `bin` directory, which stores the executables that will be run when you start Earthworm.

Also in the top-level directory is a directory containing the configuration and data files for your Earthworm instance, typically called `run_working`.  Under the `run_working` directory are directories containing your Earthworm configuration (*parameter*) files, the log files written by the modules when you run Earthworm, and a place for data files read and written by your Earthworm modules.  These directories are typically called `params`, `log`, and `data`, and their paths are stored in the `EW_PARAMS`, `EW_LOG`, and `EW_DATA_DIR` environment variables, respectively.

## The Earthworm Environment

Prior to starting Earthworm, you must run or `source` the Earthworm *environment file* appropriate to your platform.  Earthworm environment files are named `ew_*`, and are located in the `environment` directory: 

- `ew_cygwin.bash`
- `ew_linux.bash`
- `ew_linux_arm.bash`
- `ew_macos.sh`
- `ew_nt.cmd`
- `ew_sol_intel.bash`
- `ew_sol_intel.cmd`
- `ew_sol_sparc.bash`
- `ew_sol_sparc.cmd`

Note that `ew_nt.cmd` is for all Microsoft Windows operating systems.

You may copy it to your `params` directory.

Before you run or `source` your `ew_*` file, you must make sure all the directories actually exist that are specified by `EW_HOME`, `EW_VERSION`, `EW_PARAMS`, `EW_LOG`, and `EW_DATA_DIR`, and that `EW_INSTALLATION` is properly defined.

## Running Earthworm

Two key programs are the core of Earthworm: `startstop` and `statmgr`.  As its name implies, `startstop` *starts* and *stops* Earthworm according to the instructions in its configuration file, `startstop_*.d`.  The `startstop` configuration file contains the command line to run each Earthworm module, including the name of the module and its corresponding `.d` configuration file.  `statmgr` monitors the health of running Earthworm modules, as configured in its own `.d` file.  The `.desc` file for each module configures how the module is monitored and, if allowed, is killed and restarted, when necessary. 

See the module documentation in `ewdoc/WEB_DOC/index.html` for more information on configuring `startstop`, `statmgr`, and other modules.

The easiest way to get started is to run an existing Earthworm configuration, see how it works, and then modify it for your own needs.  Start with the Memphis Test Suite from <http://www.earthwormcentral.org/distribution/memphis_test.zip> or <http://folkworm.ceri.memphis.edu/ew-dist/v7.10/memphis_test.zip>.

Read the Test Suite `params/README.memphis.test` file and try running it.  Note that a minor difference in the depth value (i.e., 11.74 *versus* 11.71) is acceptable.

When the Test Suite runs properly, and the results are correct, you can request a new Installation ID from Mitch Withers at the University of Memphis.  (See the instructions at the top of `earthworm_global.d` for how to do this.  `earthworm_global.d` is distributed in the `environment` directory, but must be placed in your parameters directory to become active.)  Your Installation ID is required in your `.desc` files as well as some `.d` files.  You can modify the files in the Test Suite `params` directory to replace `INST_MEMPHIS` with your Installation ID, or you can get a fresh set of parameters from the distribution's `params` directory.  Be sure to read the `.d` files from the distribution, in case any options or their descriptions have changed.

>It is recommended that you copy the `.d` and `.desc` configuration files for the modules and other programs you need to your `params` directory from the distribution's `params` directory. 

As you've seen from the Test Suite, type `startstop` to run Earthworm.  (To have Earthworm automatically start, you can use a Unix startup script for `/etc/init.d` or you can use the StartStop Service on Windows.)

If Earthworm does not run properly, check the log files directory if the error output from `startstop` isn't helpful.  If you need help, join and then email the Google Groups' [Earthworm Community Forum](http://groups.google.com/group/earthworm_forum).

## Find Out More

If you want to learn more about Earthworm and the documentation isn't enough, or if you want to learn how to program your own Earthworm modules, ISTI offers an Earthworm class approximately yearly.  Contact ISTI to obtain the current schedule, availability, and cost for the course at <http://www.isti.com/contact>.

# COMPILING

See `src/README` for additional information.

If you want to compile the Earthworm binaries yourself, you must run or `source` the Earthworm environment file appropriate to your platform, located in the `environment` directory: 

- `ew_cygwin.bash`
- `ew_linux.bash`
- `ew_linux_arm.bash`
- `ew_macos.sh`
- `ew_nt.cmd`
- `ew_sol_intel.bash`
- `ew_sol_intel.cmd`
- `ew_sol_sparc.bash`
- `ew_sol_sparc.cmd`

Note that `ew_nt.cmd` is for all Microsoft Windows operating systems.

You do *not* need to copy it to your `params` directory.

Before you run or `source` your `ew_*` file, you must make sure the directories specified by `EW_HOME` and `EW_VERSION` exist.  Also, you may need to specify the compiler to use, especially if you're using Windows.  The directory path must match what you have.  If you don't have a Fortran compiler you won't be able to compile Hypoinverse (used to locate earthquakes), but you should be able to compile most other modules.  You may need to comment out the build of the Fortran programs (`hyp71_mgr`, `hyp2000`, `hyp2000_mgr`, `Hypo71PC`, and `rayloc_ew`) in `src/seismic_processing/Makefile`.

To compile, after you have run or sourced your environment file, run the following command, depending on your platform:

- `nmake nt`   — For Windows
- `make unix` — For the many flavors of Linux, macOS, or Solaris

# DEVELOPER WORKFLOW ON GITLAB

Earthworm development is hosted on [GitLab](http://gitlab.com) at <http://gitlab.com/seismic-software/earthworm/>:  
![GitLab Home Page](images/GitLabHomePage.png)  
>*NOTE: You must* `Sign in` *(top-right) to your Earthworm GitLab developer account to proceed.*

After you have signed in, you will be taken to the Earthworm *Project overview* page:  
![GitLab Project overview Page](images/GitLabProjectOverviewPage.png)  
>Use the sidebar on the left to navigate the Earthworm GitLab developer web site.
1. From the Earthworm GitLab developer web site, select `Issues` in the sidebar to open the *Issues* page:  
![GitLab Issues Page](images/GitLabIssuesPage.png)  
2. Select the `New issue` button at the top-right to open a *New Issue* form:  
![GitLab New Issue Form](images/GitLabNewIssueForm.png)  

	![GitLab Create Issue Button](images/GitLabCreateIssueButton.png)  
	Fill in the *New Issue* form as best you can.  You must provide a *Title*; the other fields are optional.

	Use the *Description* field to describe the issue, when it occurs or how to replicate it, and any proposed remedy or workaround.  The description should be detailed enough so another developer will understand the issue well enough to work on a solution.

	You can assign the issue to yourself or leave it unassigned.
3. When you have completed the *New Issue* form, select the `Create issue` button at the bottom to create your new issue:  
![GitLab Issue Details Page](images/GitLabIssueDetailsPage.png)  
(Select `Okay!` to dismiss the small information window near the top of the page, if it appears.)
4. Select the `Create merge request` button to open a *New merge request* form:  
![GitLab New Merge Request Form](images/GitLabNewMergeRequestForm.png)  

	![GitLab Create Merge Request Button](images/GitLabCreateMergeRequestButton.png)  
	The name of the GitLab branch for your new issue appears in the *From...* line at the top of the *New merge request* form (immediately under the form title, in bold).  You will supply the branch name later when you `git checkout` a local copy of your new branch on your own computer.

	You can assign the issue to yourself or leave it unassigned.  Any reviewers you select will be notified when updates are made.

	The *Merge options* to `Delete source branch ...` and `Squash commits ...` are recommended.
5. After you have reviewed the *New merge request* form, select the `Create merge request` button at the bottom.  This will automatically create a merge request for your new branch:  
![GitLab New Merge Request Page](images/GitLabNewMergeRequestPage.png)  

	You will use this merge request later after you fix the issue and `git commit` it to your local branch, and then `git push` your updated local branch to the Earthworm GitLab repository.
6. Select `Project overview` in the sidebar to return to the Earthworm *Project overview* page:  
![GitLab Updated Project overview Page](images/GitLabUpdatedProjectOverviewPage.png)  
	Notice the number of *Branches* for the Earthworm project has increased.
7. Select the `Clone` pull-down menu button on the right, then select the `Copy URL` icon to the right of the `Clone with HTTPS` text:  
![GitLab Clone with HTTPS](images/GitLabCloneWithHTTPS.png)  
This will copy the Earthworm GitLab repository web address (<https://gitlab.com/seismic-software/earthworm.git>) to your clipboard.
8. From a terminal window, navigate to your local top-level Earthworm development directory, such as `~/Desktop/Earthworm`, and use `git clone` to clone the Earthworm GitLab repository (paste the URL you just copied) to a local directory, such as `earthworm_git`:

	`cd ~/Desktop/Earthworm`  
	`git clone https://gitlab.com/seismic-software/earthworm.git earthworm_git`

	Provide your *username* and *password* when prompted.

	Or, you may prefer:

	`git clone https://'username:password'@gitlab.com/seismic-software/earthworm.git earthworm_git`

	to provide your GitLab *username* and *password* as part of the `git clone` command.

9. From the terminal window, `git checkout` your new branch locally:
>If you have forgotten the name of your new branch, navigate to the Earthworm GitLab *Issues* page and select your issue from the list to open the page describing your issue.  An entry will be listed under **Activity** from when you created your new branch in Step 4, above.

	For example, if your new branch is named *124-add-scnl-description-to-readme-txt*:

	`cd earthworm_git`  
	`git checkout '124-add-scnl-description-to-readme-txt'`
10. Make your changes to the source code, compile, and test that the issue is fixed.

	>Type `man gittutorial` for a brief introduction to the basic `git` command-line options.  The current `gittutorial[7]` man page can be viewed at <https://git-scm.com/docs/gittutorial>.
11. From a terminal window, use `git diff` and `git status` to view the changes to be committed:

	Show any changes that you’ve made but not yet added to the local Git index:

	`cd ~/Desktop/Earthworm/earthworm_git`  
	`git status`

	Add any missing files to the Git index:

	`git add`*`file_1 file_2 ...`*

	Show the changed/new files ready to be committed:

	`git status`

	>If you need to make any further adjustments, do so now, and then add any newly modified content to the Git index.
12. If the summary is correct, `git commit` your updates to your local branch, then `git push` your updated local branch to the Earthworm GitLab repository:

	`git commit -m`*`'add a comment here to describe the commit'`*  
	`git push`

	This will start the Earthworm GitLab build and test pipelines.

	>You may encounter a *remote: error*, such as:
	>
	>*remote: error: inflate: data stream error (too many length or distance symbols)*  
	>*remote: error: inflate returned -3*
	>
	>This is likely due to a temporary condition on the GitLab server.  Wait a short time and retry the `git push` command.
13. From the Earthworm GitLab developer web site, navigate to the *Merge requests* page and open the merge request you created in Step 5 from the list:  
![GitLab Merge Request Page](images/GitLabMergeRequestPage.png)  
14. Wait for the pipelines to complete:  
![GitLab Pilelines Passed](images/GitLabPipelinesPassed.png)

	Review the changes and verify that the pipelines have passed.  If there are problems, or either you or a reviewer decide further changes are necessary, repeat Steps 10 through 13 until you and the reviewers are satisfied that the issue is fixed.
15. When you and your reviewers are satisfied with the changes, select `Mark as ready` to merge the changes into the Earthworm *master* branch.

	>You or one of your reviewers can mark the merge request as *ready*.

	Now all your changes have been merged with the Earthworm GitLab *master* branch.

## Release Version

1. Open [Deployment/Releases](https://gitlab.com/seismic-software/earthworm/-/releases).
2. Click 'New release', fill in the required fields (Tag name should be in the version format, i.e., 7.10, 7.11, 8.0, etc.), and click 'Create release'.

GitLab will run the build, test and release pipelines.  The release pipeline creates a corresponding RPM in [Packages and Registries/Package Registry](https://gitlab.com/seismic-software/earthworm/-/packages) and build/ops containers in [Packages and Registries/Container Registry](https://gitlab.com/seismic-software/earthworm/container_registry).

## GitLab Tutorials

GitLab offers many tutorials at <https://docs.gitlab.com/ee/tutorials/>.  The [Introduction to GitLab](https://youtu.be/_4SmIyQ5eis?t=90) YouTube video on [Tutorials: Find your way around GitLab](https://docs.gitlab.com/ee/tutorials/gitlab_navigation.html) is a useful presentation of the GitLab development process, especially their use of the *Merge Request* as a container for all the information about an issue, the development branch dedicated to the issue, and ongoing updates within the branch.

# SUBVERSION USER WORKFLOW

If you are a command-line `svn` developer, open the [Earthworm GitLab developer web site](http://gitlab.com/seismic-software/earthworm/) and `Sign in` (top-right) to your Earthworm developer account, as shown above in DEVELOPER WORKFLOW ON GITLAB.

After you have signed in, you will be taken to the Earthworm *Project overview* page:  
![Subversion Project overview Page](images/SubversionProjectOverviewPage.png)  
>Use the sidebar on the left to navigate the Earthworm GitLab developer web site.
1. (GitLab Step 7)  Select the `Clone` pull-down menu button on the right, then select the `Copy URL` icon to the right of the `Clone with HTTPS` text:  
![Subversion Clone with HTTPS](images/SubversionCloneWithHTTPS.png)  
This will copy the Earthworm GitLab repository web address (<https://gitlab.com/seismic-software/earthworm.git>) to your clipboard.
2. (GitLab Step 8.)  From a terminal window, navigate to your local top-level Earthworm development directory, such as `~/Desktop/Earthworm`, and use `git clone` to clone the Earthworm GitLab repository (paste the URL you just copied) to a local directory, such as `earthworm_git`:

	`cd ~/Desktop/Earthworm`  
	`git clone https://gitlab.com/seismic-software/earthworm.git earthworm_git`

	Provide your *username* and *password* when prompted.

	Or, you may prefer:

	`git clone https://'username:password'@gitlab.com/seismic-software/earthworm.git earthworm_git`

	to provide your GitLab *username* and *password* as part of the `git clone` command.
3. (GitLab Step 1.)  From the Earthworm GitLab developer web site, select `Issues` in the sidebar to open the *Issues* page:  
![Subversion Issues Page](images/SubversionIssuesPage.png)  
4. (GitLab Step 2.)  Select the `New issue` button at the top-right to open a *New Issue* form:  
![Subversion New Issue Form](images/SubversionNewIssueForm.png)

	![Subversion Create Issue Button](images/SubversionCreateIssueButton.png)  
	Fill in the *New Issue* form as best you can.  You must provide a *Title*; the other fields are optional.

	Use the *Description* field to describe the issue, when it occurs or how to replicate it, and any proposed remedy or workaround.  The description should be detailed enough so another developer will understand the issue well enough to work on a solution.

	You can assign the issue to yourself or leave it unassigned.
5. (GitLab Step 3.)  When you have completed the *New Issue* form, select the `Create issue` button at the bottom to create your new issue:  
![Subversion Issue Details Page](images/SubversionIssueDetailsPage.png)  
(Select `Okay!` to dismiss the small information window near the top of the page, if it appears.)
6. (GitLab Step 4.)  Select the `Create merge request` button to open a *New merge request* form:  
![Subversion New Merge Request Form](images/SubversionNewMergeRequestForm.png)

	![Subversion Create Merge Request Button](images/SubversionCreateMergeRequestButton.png)    
	The name of the GitLab branch for your new issue appears in the *From...* line at the top of the *New merge request* form (immediately under the form title, in bold).  You will supply the branch name later when you `git checkout` a local copy of your new branch on your own computer.

	You can assign the issue to yourself or leave it unassigned.  Any reviewers you select will be notified when updates are made.

	The *Merge options* to `Delete source branch ...` and `Squash commits ...` are recommended.
7. (GitLab Step 5.)  After you have reviewed the *New merge request* form, select the `Create merge request` button at the bottom.  This will automatically create a merge request for your new branch:  
![Subversion New Merge Request Page](images/SubversionNewMergeRequestPage.png)

	You will use this merge request later after you fix the issue and `git commit` it to your local branch, and then `git push` your updated local branch to the Earthworm GitLab repository.
8. Select the `Check out branch` item from the`Code` pull-down menu button on the right:  
![Subversion Check Out Branch](images/SubversionCheckOutBranch.png)  
You will be provided with the `git` commands to use in the next step:  
![Subversion GIT Commands](images/SubversionGITCommands.png)  
Select the `Copy commands` icon to the right of the **Step 1** `git` commands to copy the `git` commands to your clipboard.
9. From a terminal window, navigate to your cloned Earthworm directory, such as `~/Desktop/Earthworm/earthworm_git`, and issue the **Step 1** commands *only*.

	For example, if your new branch is named *126-update-readme-md*:

	`cd ~/Desktop/Earthworm/earthworm_git`  
	`git fetch origin`  
	`git checkout -b '126-update-readme-md' 'origin/126-update-readme-md'`
10. (GitLab Step 10.)  Make your changes to the source code, compile, and test that the issue is fixed.

	>Type `man gittutorial` for a brief introduction to the basic `git` command-line options.  The current `gittutorial[7]` man page can be viewed at <https://git-scm.com/docs/gittutorial>.
11. (GitLab Step 11.)  From a terminal window, use `git diff` and `git status` to view the changes to be committed:

	Show any changes that you’ve made but not yet added to the local Git index:

	`cd ~/Desktop/Earthworm/earthworm_git`  
	`git status`

	Add any missing files to the Git index:

	`git add`*`file_1 file_2 ...`*

	Show the changed/new files ready to be committed:

	`git status`

	>If you need to make any further adjustments, do so now, and then add any newly modified content to the Git index.
12. If the summary is correct, `git commit` your updates to your local branch:

	`git commit -m`*`'add a comment here to describe the commit'`*

	After this, do *not* follow the "branch" instructions for Steps 3 and 4.  (This is because you are not allowed to merge with the *master* branch without creating a merge request.)
13. Merge the *master* branch into your branch and deal with any merge conflicts:

	`git fetch origin` # this gets the latest from the server in case anyone changed anything  
	`git merge master` # this merges *master* into your branch, *not* your branch into *master*
14. Finally, `git push` your updated local branch to the Earthworm GitLab repository:

	`git push`

	This will start the Earthworm GitLab build and test pipelines.

	>You may encounter a *remote: error*, such as:
	>
	>*remote: error: inflate: data stream error (too many length or distance symbols)*  
	>*remote: error: inflate returned -3*
	>
	>This is likely due to a temporary condition on the GitLab server.  Wait a short time and retry the `git push` command.
15. (GitLab Step 13.)  From the Earthworm GitLab developer web site, navigate to the *Merge requests* page and open the merge request you created in Step 7 from the list:  
![Subversion Merge Request Page](images/SubversionMergeRequestPage.png)
16. (GitLab Step 14.)  Wait for the pipelines to complete:  
![Subversion Pilelines Passed](images/SubversionPipelinesPassed.png)

	Review the changes and verify that the pipelines have passed.  If there are problems, or either you or a reviewer decide further changes are necessary, repeat Steps 10 through 13 until you and the reviewers are satisfied that the issue is fixed.
17. (GitLab Step 15.)  When you and your reviewers are satisfied with the changes, select `Mark as ready` to merge the changes into the Earthworm *master* branch.

	>You or one of your reviewers can mark the merge request as *ready*.

	Now all your changes have been merged with the Earthworm GitLab *master* branch.
