#
#	IMPORTANT NOTE ABOUT RUNNING CARLSUBTRIG
#
# Carlsubtrig supports the ability to use late arriving station triggers in the
# subnet trigger logic. It maintains an internal clock which is set a fixed
# number of seconds (the latency period) behind wall-clock (real, system)
# time. This latency period allows for delayed delivery of station trigger
# messages. Carlsubtrig uses this internal clock to compare to station
# trigger times. Once the station trigger-on or -off time is later than the
# internal clock time, this trigger status change is noticed by carlsubtrig. In
# order for times to be compared between machines, all machines should
# be synchronized within a few seconds or better. Xntp (available with
# Solaris2.6 and also public domain) is a good choice for time
# synchronization. 

# Because Carlsubtrig uses both system and data time to perform the
# subnet trigger logic, the system time must be set to UTC. If this is
# not done, no triggers will occur.
# Under NT, TZ must be set to GMT in ew_nt.cmd in order to run under
# GMT time.
#

#
# CarlSubTrig's Parameter File
#
#  dwilson 092308: removed noisy stations SPD, HPU, and KKH from subnets.
#
#  Basic Earthworm Setup
#
MyModuleId	MOD_CARLSUBTRIG	# Module id for this instance of CarlSubTrig -
				#
Debug		3		# Write out debug messages? (0 = No,
				#   1 = Minimal, 3 = Chatterbox )
RingNameIn	TRIG_RING	# Name of ring from which station triggers 
				#   will be read - REQUIRED.
RingNameOut	SUBTRIG_RING	# Name of ring to which triggers will be
				#   written - REQUIRED.
HeartBeatInterval	30	# Heartbeat Interval (seconds).

#
# CarlSubTrig Specific Setup
#
StationFile	"ewconfig/chan/carlsta.scnl"	  # Name of file containing station information -
				        #   REQUIRED.
Latency	    150		  	  # Number of seconds that the Network clock
				        #   is behind wall clock REQUIRED.
NetTriggerDur   10		  # Number of seconds for the base network
				        #   trigger duration REQUIRED.
SubnetContrib   15		  # Addition contribution to network trigger
				        #   duration for each subnet that triggered
                                #   REQUIRED.
PreEventTime    15		  # Number of seconds added to beginning of
				        #   network trigger REQUIRED.
MaxDuration     500		  # Maximum duration allowed for network trigger
DefStationDur   120	 	  # Default number of seconds for station
				        #   trigger duration if the trigger-off 
				        #   message is not received. REQUIRED.
ListSubnets     2               # Flag to list untriggered stations
                                #   =0 or command not present: list all 
                                #        triggered stations 
                                #   =1 list all stations in triggered subnets
                                #   =2 list all stations in triggered subnets 
                                #        plus any other triggered stations. 
                                #   =3 list all stations in subnets that had
                                #        any stations triggered
AllSubnets      3              # If this many subnets trigger, put wildcard
                                #   SCN in event message

# paulf commented this out for CISN testing
#CompAsWild                      # Flag (no value) to list component names in
                                #   trigger messages as `*' (wildcard).
MaxTrigMsgLen   30000           # maximum length (bytes) of a triglist message;
                                #   up to MAX_BYTES_PER_EW (earthworm.h).

# Load the next valid trigger sequence number
@trig_id.d            

# List the message logos to grab from transport ring
#              Installation       Module          Message Types (hard-wired)
GetEventsFrom  INST_WILDCARD    MOD_WILDCARD    # TYPE_CARLSTATRIG # REQUIRED.

# Non-seismic or other channels that should be included in all event messages
# List one SCN per line, as many as you need
# paulf commented this out for trigger coordinator action
#Channel  *.T.HV.--

# Subnet definitions for the CarlSubTrig Earthworm module
# Each Subnet must be on a single line
# Subnet        Minimum to  List of Station.Component.Network
# Name          Trigger     Codes (space delimited)
# -------------- ---    -------------------------------------------

Subnet 1  3  HLK.HHZ.PT.-- HPUD.EHZ.HV.-- HUAD.HHZ.HV.-- KKUD.EHZ.HV.-- KOHD.EHZ.HV.-- WAID.EHZ.HV.-- | 
Subnet 2  3  HPUD.EHZ.HV.-- HSSD.HHZ.HV.-- HUAD.HHZ.HV.-- KKUD.EHZ.HV.-- WAID.EHZ.HV.-- | 
Subnet 3  3  CACD.EHZ.HV.-- HUAD.HHZ.HV.-- KAAD.EHZ.HV.-- KHLU.HHZ.PT.-- KKH.EHZ.PT.-- TOUO.HHZ.HV.-- | 
Subnet 4  3  ALEP.EHZ.HV.-- DAND.EHZ.HV.-- ELEP.EHZ.HV.-- HSSD.HHZ.HV.-- MLOD.HHZ.HV.-- MOKD.HHZ.HV.-- PLAD.EHZ.HV.-- RCOD.EHZ.HV.-- SWRD.EHZ.HV.-- TOUO.HHZ.HV.-- TRAD.EHZ.HV.-- WILD.EHZ.HV.-- | 
Subnet 5  2  ALEP.EHZ.HV.-- HPUD.EHZ.HV.-- HSSD.HHZ.HV.-- MLOA.HHZ.PT.-- MLOD.HHZ.HV.-- PLAD.EHZ.HV.-- | 
Subnet 6  3  HILB.HHZ.PT.-- HSSD.HHZ.HV.-- R931.HHZ.HV.-- JOKA.HHZ.HV.-- KKUD.EHZ.HV.-- NAGD.EHZ.HV.-- PLAD.EHZ.HV.-- | 
Subnet 7  3  ERZ3.HHZ.HV.-- ERZ4.HHZ.HV.-- R931.HHZ.HV.-- JOKA.HHZ.HV.-- KIND.HHZ.HV.-- KLUD.EHZ.HV.-- KUPD.EHZ.HV.-- NAGD.EHZ.HV.-- P07.EHZ.HV.-- | 
Subnet 8  3  DEVL.HHZ.HV.-- JCUZ.HHZ.HV.-- R931.HHZ.HV.-- JOKA.HHZ.HV.-- KNHD.EHZ.HV.-- KUPD.EHZ.HV.-- NPOC.HHZ.HV.-- PAUD.HHZ.HV.-- STCD.HHZ.HV.-- | 
Subnet 9  3  AHUD.EHZ.HV.-- DEVL.HHZ.HV.-- KNHD.EHZ.HV.-- NAHU.HHZ.HV.-- PAUD.HHZ.HV.-- PUHI.HHZ.HV.-- RIMD.HHZ.HV.-- | 
Subnet 10  3  AHUD.EHZ.HV.-- HLPD.HHZ.HV.-- HTCD.EHZ.HV.-- KAED.EHZ.HV.-- KNHD.EHZ.HV.-- PAUD.HHZ.HV.-- POLD.EHZ.HV.-- STCD.HHZ.HV.-- | 
Subnet 11  4  AHUD.EHZ.HV.-- BYL.HHZ.HV.-- HAT.HHZ.HV.-- KKO.HHZ.HV.-- NAHU.HHZ.HV.-- OBL.HHZ.HV.-- OTLD.HHZ.HV.-- RIMD.HHZ.HV.-- RSDD.HHZ.HV.01 SBL.HHZ.HV.-- SDH.HHZ.HV.-- UWB.HHZ.HV.-- UWE.HHZ.HV.-- WRM.HHZ.HV.-- | 
Subnet 12  3  AIND.HHZ.HV.-- CPKD.EHZ.HV.-- DESD.EHZ.HV.-- HLPD.HHZ.HV.-- HTCD.EHZ.HV.-- OTLD.HHZ.HV.-- WOOD.EHZ.HV.-- | 
Subnet 13  3  AIND.HHZ.HV.-- CPKD.EHZ.HV.-- DESD.EHZ.HV.-- MLOD.HHZ.HV.-- TRAD.EHZ.HV.-- | 
Subnet 14  3  DESD.EHZ.HV.-- HLPD.HHZ.HV.-- HTCD.EHZ.HV.-- KHU.HHZ.PT.-- PPLD.HHZ.HV.-- WOOD.EHZ.HV.-- | 
Subnet 15  3  AIND.HHZ.HV.-- DAND.EHZ.HV.-- HOVE.HHZ.HV.-- KAAD.EHZ.HV.-- KHU.HHZ.PT.-- RCOD.EHZ.HV.-- TRAD.EHZ.HV.-- WOOD.EHZ.HV.-- | 
Subnet 16  3  HOVE.HHZ.HV.-- KAAD.EHZ.HV.-- KHU.HHZ.PT.-- PPLD.HHZ.HV.-- SPDD.EHZ.HV.-- | 
Subnet 17  3  HLK.HHZ.PT.-- KHLH.HHZ.PT.-- KKUD.EHZ.HV.-- KOHD.EHZ.HV.-- OPA.HHZ.PT.-- WMR.EHZ.PT.-- | 
Subnet 18  3  DHH.EHZ.PT.-- HON.HHZ.PT.-- LIH.HHZ.PT.-- OPA.HHZ.PT.-- WMR.EHZ.PT.-- | 
Subnet 19  2  KEKH.HHZ.PT.-- LIH.HHZ.PT.-- OPA.HHZ.PT.-- | 
