# 
#  Configuration file to export Earthworm events to SeisComP
#
# The period of our local heartbeat (HeartBeatInt) must be safely smaller
# (faster) than our advertised period in our .desc file (tsec:).
# Otherwise we'll get continually restarted for no good reason.
# Note that tsec:0 implies no heartbeats expected, and so we'll never get
# restarted.
#
# The rate at which we send socket heartbeats to our distant partner should
# be considerably faster than the rate at which our partner expects them.
# Otherwise, a heartbeat delayed in transmission will cause our partner to
# conclude that the link is broken, and cause them to break the link and
# reinitialize. Which will cause us to do the same.
#
# For export, the ServerIPAdr is the address of the port to be used in
# the exporting machine.  This is to specify a network card case the
# exporting machine has several network cards.
# Using ServerIPAdr 0.0.0.0 will allow export to answer connections
# on any valid address of the machine.
#
# If SocketTimeout is specified, it should be at least as large as the
# expected period of heartbeats from our distant partner.

# Basic Earthworm setup
#----------------------
 MyModuleId     MOD_EXPORT_E        # module id for this program
 RingName       HYPO_RING           # transport ring to use for input/output
 HeartBeatInt   30                  # EW internal heartbeat interval (sec)
                                    #   Should be >= RcvAliveInt
# Logging Control
#----------------
 LogFile        1      # If 0, don't write logfile
                       #    1, write to logfile and stdout
                       #    2, write to module log but not stderr/stdout
#Verbose               # If uncommented, VERY LARGE logfiles will be
                       #   generated with info about queue status of
                       #   each msg, socket alive msgs sent & received.

# Logos of messages to export to client systems.
#------------------------------------------------
# Installation and Module can be wildcards.
# Knows how to decipher these message types:
#  w/  location code: TYPE_TRACEBUF2, TYPE_TRACE2_COMP_UA,
#                     TYPE_PICK_SCNL, TYPE_CODA_SCNL
#  w/o location code: TYPE_TRACEBUF, TYPE_TRACE_COMP_UA,
#                     TYPE_PICK2K, TYPE_CODA2K msgs
#
#              Installation       Module       Message Type
#--------------------------------------------------------------
 GetMsgLogo       ${EW_INST_ID}     MOD_WILDCARD   TYPE_HYP2000ARC

# Set up Message Queues
# Note: socket "alive" messages are not put in either queue
#----------------------------------------------------------
MaxMsgSize             16384   # maximum size (bytes) for input/output msgs
RingSize                1000   # number of msgs in MessageStacking queue.
                               #  (msgs waiting to be sent to socket)
SendQueueLength          254   # Optional command: #msgs in SendQueue
                               #  (msgs sent, acknowledgments pending)
                               #  Valid range: 1-254 (default 100)
                               #  shipped on export restarts.
MaxLatency                60   # Optional command: Packets with delay time
                               #  (i.e., time between present and packet
                               #  start time) greater than this amount will be
                               #  filtered out. Time is in minutes.  0 -> time
                               #  is not checked. System time must be accurate.
                               #  Only works on waveform messages.

# Set up server/client communications
#------------------------------------
 ServerIPAdr  130.118.86.175   # Listen for connections on this IPaddress.
                               #  0.0.0.0 will use any address on machine.
 ServerPort            64014   # Well-known port number to export msgs on

 SendAliveText       "alive"   # string sent to client as heartbeat
 SendAliveInt             30   # seconds between alive msgs sent to client.
                               #  0 => no heartbeat
 RcvAliveText        "alive"   # text of client's heartbeat (we get this)
 RcvAliveInt             150   # seconds between client's heartbeats to us.
                               #  0 => no heartbeat

# Optional Socket commands:
# SocketTimeout defaults to RcvAliveInt + 3
#  If set to -1, all socket calls will block (no timeout).
#  SocketTimeout has no effect in export, unless it is set to -1,
#  because there is no code in export to handle socket timeouts.
#  If set to -1, the program may run slightly more efficiently because
#  timeout checking code will not execute.
#----------------------------------------------------------------------
SocketTimeout 400000  # Timeout length in milliseconds for socket calls

SocketDebug   0       # if 1, socket_ew debug statements are logged
                      # if 0, socket_ew debug is turned off (default)

