#include "config.h"
#include "kom.h"
#include "ringplayer.h"
#include "strfuncs.h"
#include "time_ew_common.h"
#include <errno.h>
#include <string.h>

#define ncommand 1

static int process_com(char *com, char *value, PARAMS *params) {
  char timestr[DATESTR_MAX];
  int retcode = 0;
  if (strcmp(com, "Binary") == 0) {
    if (value) {
      params->Binary = atoi(value);
      if (params->Binary)
        logit("o", "Binary=%d\n", params->Binary);
    }
  } else if (strcmp(com, "Debug") == 0) {
    if (value) {
      params->Debug = atoi(value);
      if (params->Debug)
        logit("o", "Debug=%d\n", params->Debug);
    }
  } else if (strcmp(com, "LogFile") == 0) {
    if (value) {
      params->LogFile = atoi(value);
      if (params->Debug)
        logit("o", "LogFile=%d\n", params->LogFile);
    }
  } else if (strcmp(com, "Latencyfile") == 0) {
    if (value) {
      strncpy(params->Latencyfile, value, sizeof(params->Latencyfile));
      if (params->Debug)
        logit("o", "Latencyfile=%s\n", params->Latencyfile);
    }
  } else if (strcmp(com, "LatencyfileExt") == 0) {
    if (value) {
      strncpy(params->LatencyfileExt, value, sizeof(params->LatencyfileExt));
      if (params->Debug)
        logit("o", "LatencyfileExt=%s\n", params->LatencyfileExt);
    }
  } else if (strcmp(com, "Outputfile") == 0) {
    if (value) {
      strncpy(params->Outputfile, value, sizeof(params->Outputfile));
      if (params->Debug)
        logit("o", "Outputfile=%s\n", params->Outputfile);
    }
  } else if (strcmp(com, "OutputfileExt") == 0) {
    if (value) {
      strncpy(params->OutputfileExt, value, sizeof(params->OutputfileExt));
      if (params->Debug)
        logit("o", "OutputfileExt=%s\n", params->OutputfileExt);
    }
  } else if (strcmp(com, "Mseeddir") == 0) {
    if (value) {
      int index = strlen(value) - 1;
      if (value[index] == '/') {
        value[index] = '\0';
      }
      strncpy(params->Mseeddir, value, sizeof(params->Mseeddir));
      if (params->Debug)
        logit("o", "Mseeddir=%s\n", params->Mseeddir);
    }
  } else if (strcmp(com, "ArchiveFormat") == 0) {
    if (value) {
      params->ArchiveFormat = atoi(value);
      if (params->Debug)
        logit("o", "ArchiveFormat=%d\n", params->ArchiveFormat);
    }
  } else if (strcmp(com, "Pinno") == 0) {
    if (value) {
      params->Pinno = atoi(value);
      if (params->Debug)
        logit("o", "Pinno=%d\n", params->Pinno);
    }
  } else if (strcmp(com, "StartTime") == 0) {
    if (value) {
      parsetime_ew(&params->StartTime, value);
      datestriso(params->StartTime, timestr, sizeof(timestr));
      if (params->Debug)
        logit("o", "StartTime=%s (%f)\n", timestr, params->StartTime);
    }
  } else if (strcmp(com, "EndTime") == 0) {
    if (value) {
      parsetime_ew(&params->EndTime, value);
      datestriso(params->EndTime, timestr, sizeof(timestr));
      if (params->Debug)
        logit("o", "EndTime=%s (%f)\n", timestr, params->EndTime);
    }
  } else if (strcmp(com, "LatencyInterval") == 0) {
    if (value) {
      params->LatencyInterval = atof(value);
      if (params->Debug)
        logit("o", "LatencyInterval=%f\n", params->LatencyInterval);
    }
  } else if (strcmp(com, "LatencyOutputType") == 0) {
    if (value) {
      params->LatencyOutputType = atoi(value);
      if (params->Debug)
        logit("o", "LatencyOutputType=%d\n", params->LatencyOutputType);
    }
  } else if (strcmp(com, "MatchAllChannelCodes") == 0) {
    params->MatchAllChannelCodes = atoi(value);
    if (params->Debug)
      logit("o", "MatchAllChannelCodes=%d\n", params->MatchAllChannelCodes);
  } else if (strcmp(com, "MatchAllLocationCodes") == 0) {
    params->MatchAllLocationCodes = atoi(value);
    if (params->Debug)
      logit("o", "MatchAllLocationCodes=%d\n", params->MatchAllLocationCodes);
  } else if (strcmp(com, "SCNLfile") == 0) {
    char buffer[MAX_SIZE]; // Buffer for file contents
    SCNL_ENTRY *entry;
    FILE *infp = fopen(value, "r");
    int linenum;
    if (infp == NULL) {
      logit("e", "could not open file (%s): %s\n", value, strerror(errno));
      retcode = 2;
      return retcode;
    }
    scnl_table_size(&params->scnl_table, infp);
    if (params->Debug)
      logit("o", "SCNLfile=%s\n", value);
    for (linenum = 1;; linenum++) {
      if (fgets(buffer, sizeof(buffer), infp) == NULL) {
        break;
      }
      entry = scnl_table_add(&params->scnl_table);
      scnl_table_set_entry(entry, buffer);
      if (params->Debug)
        logit("o", "SCNLfile (line %d) %s %s %s %s\n", linenum, entry->sta,
              entry->chan, entry->net, entry->loc);
    }
    if (linenum == 0) {
      logit("e", "SCNLfile: could not read any lines from %s\n", value);
      retcode = 4;
    }
  } else {
    logit("e", "Unexpected command: %s\n", com);
    retcode = 1;
  }
  return retcode;
}

static int process_arg(char *arg, PARAMS *params) {
  char *com;
  char *value;
  if (arg[0] != '-' || arg[1] != '-') {
    return 8;
  }
  com = arg + 2;
  value = strchr(com, '=');
  if (value == NULL) {
    return 0x10;
  }
  *value = '\0';
  value++;
  return process_com(com, value, params);
}

void readConfigFile(char *progname, char *filename, PARAMS *params) {
  char *com;
  int nfiles;
  int success;
  /* Open the main configuration file
   **********************************/
  nfiles = k_open(filename);
  if (nfiles == 0) {
    logit("e", "%s: Error opening command file <%s>; exiting!\n", progname,
          filename);
    exit(-1);
  }

  /* Process all command files
   ***************************/
  while (nfiles > 0) /* While there are command files open */
  {
    while (k_rd()) /* Read next line from active file  */
    {
      com = k_str(); /* Get the first token from line */

      /* Ignore blank lines & comments
       *******************************/
      if (!com)
        continue;
      if (com[0] == '#')
        continue;

      /* Open a nested configuration file
       **********************************/
      if (com[0] == '@') {
        success = nfiles + 1;
        nfiles = k_open(&com[1]);
        if (nfiles != success) {
          logit("e", "%s: Error opening command file <%s>; exiting!\n",
                progname, &com[1]);
          exit(-1);
        }
        continue;
      }

      /* Process anything else as a command
       ************************************/
      if (process_com(com, k_str(), params)) {
        // Command is not valid
        logit("e", "%s: <%s> unknown command in <%s>.\n", progname, com,
              filename);
        continue;
      }

      /* See if there were any errors processing the command
       *****************************************************/
      if (k_err()) {
        logit("e", "%s: Bad <%s> command in <%s>; exiting!\n", progname, com,
              filename);
        exit(-1);
      }
    }
    nfiles = k_close();
  }
}

/**
 * Process the configuration items
 * @param argc the number of arguments
 * @param argv the array of arguments
 * @param params the parameters
 * @param argi the first argument to process, 0 to read the configuration file
 * and then process other arguments, otherwise the starting index  if there is
 * no configuration file the starting index
 * @return 0 if sucess, positive number if error.
 */
int config(int argc, char *argv[], PARAMS *params, int argi) {
  int retcode = 0;
  if (argi == 0) {
    readConfigFile(argv[0], argv[1], params);
    argi = 2;
  }
  // process any other args
  for (; argi < argc; argi++) {
    retcode = process_arg(argv[argi], params);
    if (retcode) {
      // Command is not valid
      logit("e", "%s: <%s> unknown command\n", argv[0], argv[argi]);
      break;
    }
  }
  return retcode;
}

/**
 * Initialize the parameters to the default values.
 */
void initParams(PARAMS *params) {
  // Set all default parameters
  memset(params, 0, sizeof(PARAMS));
  params->LatencyInterval = 1.0;
  params->Binary = 1; // default to binary
  params->LatencyOutputType = LATENCY_OUTPUT_TEXT_UTC;
}

/**
 * Print config help.
 */
void printConfigHelp(FILE *fp) {
  fprintf(fp, "• Binary [0 for text, 1 for binary]\n");
  fprintf(fp, "• Debug [0 for no debug messages, higher number for more "
              "information]\n");
  fprintf(fp, "• Latencyfile [the path to the latency file or the directory "
              "with latency files]\n");
  fprintf(fp, "• LatencyfileExt [file extension to match when Latencyfile is a "
              "directory]\n");
  fprintf(fp, "• Outputfile [the path to the output the fileplayer input file "
              "or the directory when Latencyfile is a directory]\n");
  fprintf(fp, "• OutputfileExt [output file extension to add to the output "
              "file name]\n");
  fprintf(fp, "• Mseeddir [The directory of the miniSEED files or miniSEED "
              "database file.]\n");
  fprintf(fp, "• ArchiveFormat [The format of the miniSEED file structure "
              "(0=BUD, 1=Seiscomp, 2=mseedindex, default is 0.)]\n");
  fprintf(fp, "• ChannelCode [The channel code used for latency, default is "
              "the actual channel code in latency.]\n");
  fprintf(fp, "• LocationCode [The location code used for latency, default is "
              "the actual location code in latency.]\n");
  fprintf(fp, "• Pinno [The pin number for TRACEBUF2, default is 0.]\n");
  fprintf(fp,
          "• StartTime [The start time for all data that will be transmitted. "
          "This may be a date string (ISO format, 2021-01-12T21:49:23.0) or a "
          "floating point epoch in seconds (seconds since Jan 1, 1970.) The "
          "default is the start time in the latency data.]\n");
  fprintf(fp, "• EndTime [The end time for all data that will be transmitted. "
              "This may be a date string (ISO format, 2021-01-12T21:49:23.0) "
              "or a floating point epoch in seconds (seconds since Jan 1, "
              "1970.)  The default is the end time in the latency data.]\n");
  fprintf(fp,
          "• LatencyInterval [The interval for the latency data in seconds, "
          "default to 1 second. This is needed when the start time and end "
          "time are not both provided (they are the same.)]\n");
  fprintf(
      fp,
      "• LatencyOutputType [The latency output type for -d mode ((0 for "
      "LATENCY_SCNL_BIN, 1 for LATENCY_SCNL_TEXT, 2 for text with epoch, 3 "
      "for text with UTC, 4 for text with local time zone, default is 3.)]\n");
}
