
/********************************************************************
 *                    reader - a debugging tool                     *
 *                                                                  *
 ********************************************************************/

/*
 *
 * Copyright (c) 2017 California Institute of Technology.
 *
 * All rights reserved.
 *
 * This program is distributed WITHOUT ANY WARRANTY whatsoever.
 *
 * Do not redistribute this program without written permission.
 *
 */

#include "ringplayer.h"
#include <earthworm.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <transport.h>
#include <unistd.h>

int main(int argc, char *argv[]) {
  int fd;
  char fileName[1024];
  char buffer[MAX_SIZE];
  int nread = 0;

  // Preferences - TODO: Offload to config file?
  int printHeader = 1;
  int ignoreHeartbeats = 1;
  int ignoreErrorMsg = 1;
  int ignoreWildcards = 0;
  int ignoreRestarts = 0;
  int ignoreReqstatus = 0;
  int ignoreStatusMsg = 0;
  Header header;

  if ((argc != 2)) {
    printf("Error: argc != 2\n");
    return -1;
  }

  strcpy(fileName, argv[1]);

  fd = open(fileName, O_RDONLY);
  lseek(fd, 0, SEEK_SET);

  /*
  read(fd, buffer, sizeof(int64_t));
  printf("sec:%d ",(int) buffer);

  read(fd, buffer, sizeof(int64_t));
  printf("usec:%d ", (int) buffer);

  read(fd, buffer, sizeof(int64_t));
  printf("length:%d ", (int) buffer);

  read(fd, buffer, sizeof(char));
  printf("type:%c ", (char) buffer);

  read(fd, buffer, sizeof(char));
  printf("mod:%c ", (char) buffer);

  read(fd, buffer, sizeof(char));
  printf("instid:%c ", (char) buffer);

  read(fd, buffer, 1024);
  printf("message:%s\n", (char *) buffer);
  */

  while (1) {
    read(fd, &header, sizeof(header));

    nread = read(fd, buffer, header.length);

    buffer[header.length] = '\0';

    if (nread <= 0)
      break;

    if (((int)header.type == TYPE_WILDCARD) && (ignoreWildcards == 1)) {
      continue;
    }
    if (((int)header.type == TYPE_ERROR) && (ignoreErrorMsg == 1)) {
      continue;
    }
    if (((int)header.type == TYPE_HEARTBEAT) && (ignoreHeartbeats == 1)) {
      continue;
    }
    if (((int)header.type == TYPE_RESTART) && (ignoreRestarts == 1)) {
      continue;
    }
    if (((int)header.type == TYPE_REQSTATUS) && (ignoreReqstatus == 1)) {
      continue;
    }
    if (((int)header.type == TYPE_STATUS) && (ignoreStatusMsg == 1)) {
      continue;
    }

    if (printHeader) {
      printf("[%d]", (int)header.sec);
      printf("[%d]", (int)header.usec);
      printf("[%d]", (int)header.length);
      printf("[%d]", (int)header.type);
      printf("[%d]", (int)header.mod);
      printf("[%d]", (int)header.instid);
    }
    printf("%s", (char *)buffer);

    // if(((int) header.type == 19) || ((int) header.type == 20))
    { printf("\n"); }
  }

  close(fd);

  return 0;
}
