#include "callbacks.h"
#include "config.h"
#include "libseed.h"
#include "libsupport.h"
#include "libmsgs.h"
#include "libseed.h"
#include "options.h"

#include "ew_globals.h"
#include "logging.h"
#include "trace_buf.h"
#include "q660_globals.h"
#include "ew_globals.h"

#ifdef ENDIAN_LITTLE
  #define TH2_DATATYPE "i4"
#else
  #define TH2_DATATYPE "s4"
#endif

static BOOLEAN print_filename = FALSE ;
static BOOLEAN write_filename = FALSE ;

#ifndef X86_WIN32
#include <sys/stat.h>
#endif

void q660_poc_handler (enum tpocstate pocstate, tpoc_recvd *poc_recv)
{
  tpocmsg pocmsg ;
  tpar_create *pcret ;
  string s, s1 ;
  int i ;

  pcret = &gConfig.par_create ;
  if ((cfgphase == CFG_RUN) &&
         (poc_recv->msg.q660_sn[0] == pcret->q660id_serial[0]) &&
         (poc_recv->msg.q660_sn[1] == pcret->q660id_serial[1]))
      {
        pocmsg.new_ip_address = poc_recv->ip_address ;
        memcpy (&pocmsg.new_ipv6_address, &poc_recv->ip6_address, sizeof(tip_v6)) ;
        pocmsg.ipv6 = poc_recv->ipv6 ;
        pocmsg.new_base_port = poc_recv->msg.baseport ;
        pocmsg.poc_token = poc_recv->msg.token ;
        i = poc_recv->msg.baseport ;
        if (pocmsg.ipv6)
            sprintf(s, "%s:%u", inet_ntop6 (&poc_recv->ip6_address), i) ;
          else
            sprintf(s, "%s:%u", showdot(poc_recv->ip_address, s1), i) ;
        strcpy(pocmsg.log_info, s) ;
        lib_poc_received (context, &pocmsg) ;
      }
}

static void ready_to_run (void)
{
  lib_get_dpcfg (context, &dpcfg) ;
  lib_change_state (context, LIBSTATE_RUN, LIBERR_NOERR) ;
}

static void clear_globals (void)
{

  previous_state = LIBSTATE_IDLE ;
  state_change = FALSE ;
  cfg_change = FALSE ;
  screen_busy = FALSE ;
}

void q660_state_callback (pointer p)
{
  tstate_call *ps ;
  enum tliberr err ;

  ps = p ;
  switch (ps->state_type) {
    case ST_STATE :
      current_state = (enum tlibstate)ps->info ;
      if (current_state != previous_state)
          switch (current_state) {
            case LIBSTATE_RUN :
              err = lib_get_sysinfo (context, &local_sysinfo, &local_seed) ;
              if (err)
                  {
                    memclr (&local_sysinfo, sizeof(tsysinfo)) ;
                    memclr (&local_seed, sizeof(tseed)) ;
                  }
              break ;
            case LIBSTATE_RUNWAIT :
              ready_to_run () ;
              break ;
            case LIBSTATE_IDLE :
            case LIBSTATE_WAIT :
              clear_globals () ;
              lib_get_state (context, &err, &opstat) ;
              break ;
            case LIBSTATE_TERM :
              if (context)
                  lib_destroy_context (&context) ;
              cfgphase = CFG_IDLE ;
              gConfig.par_register.opt_start = OST_LAST ; /* probably have a save time now */
              break ;
            default :
              break ;
          } ;
      previous_state = current_state ;
      break ;
    default :
      break ;
  }
}

void q660_msgs_callback (pointer p)
{
  string s;
  tmsg_call *pm = p;
  *s = '\0';
  lib_get_msg(pm->code, s); // get message for code
  logMessage("q660msg %s %s\n", s, pm->suffix);
}

enum tfilekind {FK_UNKNOWN, FK_CFG, FK_CONT, FK_IDXDAT} ;

static enum tfilekind translate_file (const pchar s, pchar res)
{
  enum tfilekind retval = FK_UNKNOWN;
  const char *pc = strchr(s, '/');
  *res = '\0';  // default to empty
  if (!pc)
  {
    // No drive means a data file
    retval = FK_IDXDAT;
    pc = s;
  }
  else
  {
    switch (pc - s)
    {
      case 3:
        if (strncmp(s, "cfg", 3) == 0)
        {
          retval = FK_CONT;
        }
        break;
      case 4:
        if (strncmp(s, "cont", 4) == 0)
        {
          retval = FK_CFG;
        }
        break;
    }
    ++pc; // skip '/'
  }
  if (retval != FK_UNKNOWN)
  {
    strcpy(res, gConfig.ContFileDir);
    strcat(res, pc);
  }
  return retval;
}

// File access callback
/* Extremely minimal implementation. Doesn't check return type from translate_file,
   but as a result cont.bin works as well as cont/cont.bin */
void q660_file_callback (pointer p)
{
  tfileacc_call *pfa ;
  string fname ;
#ifdef X86_WIN32
  U32 rwmode, openmode ;
  unsigned long numread ;
  unsigned long numwrite ;
#else
  int flags ;
  mode_t rwmode ;
  off_t result, long_offset ;
  int numread ;
  int numwrite ;
  struct stat sb ;
#endif

  pfa = p ;
  pfa->fault = FALSE ;
  switch (pfa->fileacc_type) {
    case FAT_OPEN :
      if (translate_file (pfa->fname, fname) == FK_UNKNOWN)
          {
            pfa->fault = TRUE ;
            return ;
          }
#ifdef X86_WIN32
      if (pfa->options & LFO_CREATE)
          openmode = CREATE_ALWAYS ;
        else
          openmode = OPEN_EXISTING ;
      if (pfa->options & LFO_WRITE)
          rwmode = GENERIC_WRITE ;
        else
          rwmode = 0 ;
      if (pfa->options & LFO_READ)
          rwmode = rwmode | GENERIC_READ ;
      pfa->handle = CreateFile (fname, rwmode, 0, NULL, openmode, FILE_ATTRIBUTE_NORMAL, 0) ;
      if (pfa->handle == INVALID_HANDLE_VALUE)
          pfa->fault = TRUE ;
#else
      if (pfa->options & LFO_CREATE)
          flags = O_CREAT | O_TRUNC ;
        else
          flags = 0 ;
      if (pfa->options & LFO_WRITE)
          if (pfa->options & LFO_READ)
              flags |= O_RDWR ;
            else
              flags |= O_WRONLY ;
      else if (pfa->options & LFO_READ)
          flags |= O_RDONLY ;
      rwmode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH ;
      pfa->handle = open (fname, flags, rwmode) ;
      if (pfa->handle < 0)
          {
            pfa->handle = INVALID_FILE_HANDLE ;
            pfa->fault = TRUE ;
          }
#endif
      break ;
    case FAT_CLOSE :
#ifdef X86_WIN32
      CloseHandle (pfa->handle) ;
#else
      close (pfa->handle) ;
#endif
      break ;
    case FAT_DEL :
      if (translate_file (pfa->fname, fname) == FK_UNKNOWN)
          {
            pfa->fault = TRUE ;
            return ;
          }
#ifdef X86_WIN32
      DeleteFile (fname) ;
#else
      remove (fname) ;
#endif
      break ;
    case FAT_SEEK :
#ifdef X86_WIN32
      if (SetFilePointer (pfa->handle, pfa->options, NULL, FILE_BEGIN) == 0xFFFFFFFF)
          pfa->fault = TRUE ;
#else
      long_offset = pfa->options ;
      result = lseek(pfa->handle, long_offset, SEEK_SET) ;
#ifdef __APPLE__
      if (result < 0)
          pfa->fault = TRUE ;
#else
      if (result != long_offset)
          pfa->fault = TRUE ;
#endif
#endif
      break ;
    case FAT_READ :
#ifdef X86_WIN32
      ReadFile (pfa->handle, pfa->buffer, pfa->options, &numread, NULL) ;
      if ((int)numread != pfa->options)
          pfa->fault = TRUE ;
#else
      numread = (int)read (pfa->handle, pfa->buffer, pfa->options) ;
      if (numread != pfa->options)
          pfa->fault = TRUE ;
#endif
      break ;
    case FAT_WRITE :
#ifdef X86_WIN32
      WriteFile (pfa->handle, pfa->buffer, pfa->options, &numwrite, NULL) ;
      if ((int)numwrite != pfa->options)
          pfa->fault = TRUE ;
#else
      numwrite = (int)write(pfa->handle, pfa->buffer, pfa->options) ;
      if (numwrite != pfa->options)
          pfa->fault = TRUE ;
#endif
      break ;
    case FAT_SIZE :
#ifdef X86_WIN32
      pfa->options = GetFileSize (pfa->handle, NULL) ;
#else
      fstat (pfa->handle, &sb) ;
      pfa->options = (int)sb.st_size ;
#endif
      break ;
    default :
      break ; /* Remainder of cases only required for balelib support */
  }
}

void process_rate(TracePacket *ptp, double rate, int nsamp, double timestamp)
{
  // handle the sub 1hz channels differently
  if(rate > 0) {
    if (nsamp == 0)
    {
       nsamp = rate;
    }
  } else {
    if (nsamp == 0)
    {
       nsamp = 1;
    }
    rate = -1.0 * ((double)1.0 / rate);
  }
  ptp->trh2.nsamp = nsamp;
  ptp->trh2.samprate = rate;
  ptp->trh2.starttime = JANfIRST2016 + timestamp;
  ptp->trh2.endtime = ptp->trh2.starttime + ((ptp->trh2.nsamp-1) / ptp->trh2.samprate);
}

size_t process_station_name(TracePacket *ptp, char *station_name, char *channel, char *location)
{
    size_t size;
    // seperate the station from the net <network>-<station>
    char *pnet = station_name;
    char *psta = strchr(pnet, '-');
    if (psta == NULL) { // if no network
      size = 0;
      psta = pnet;
    } else {
      size = psta - pnet;
      strncpy(ptp->trh2.net, pnet, size);
      psta++;
    }
    ptp->trh2.net[size] = '\0';
    strcpy(ptp->trh2.sta, psta);

    strcpy(ptp->trh2.chan, channel);
    // if our location is "  " or "", make it "--"
    if( (location[0] == ' ' && location[1] == ' ') ||
      location[0] == '\0') {
      strcpy(ptp->trh2.loc, "--");
    } else {
      strcpy(ptp->trh2.loc, location);
    }
    return size;
}

void q660_mini_callback (pointer p)
{
  pminiseed_call pm ;

  if (!gConfig.send_mini)
    return;

  pm = p ;
  if (pm->miniseed_action == MSA_512)
  {
    if (testflag & TEST_PRINTDATA)
    {
      TracePacket tp;
      process_station_name(&tp, pm->station_name, pm->channel, pm->location);
      printf ("MS %-10s %s %s %s %s\n", pm->station_name, tp.trh2.sta, tp.trh2.chan, tp.trh2.net, tp.trh2.loc);
    }
    else
    {
      // put the data into the ring
      EW_DataLogo.type = TypeMseed;
      if(tport_putmsg(&EW_MseedRegion, &EW_DataLogo, LIB_REC_SIZE, pm->data_address) != PUT_OK) {
        logMessage("XXX Error putting MSEED packet into ring\n");
      }
    }
  }
}

static string MSA_DESC[] =
{
"512",
"ARC",
"FIRST",
"INC",
"FINAL",
"GETARC",
"RETARC"
} ;

void q660_amini_callback (pointer p)
{
  pminiseed_call pm ;
  string fname ;
  string s, s1 ;
  tfile_handle afile ;
  int size ;
  pchar minus ;
  BOOLEAN bad ;

  if (gConfig.write_archive == 0)
      return ;
  pm = p ;
  strcpy(s1, pm->station_name) ;
  minus = strpbrk(s1, "-") ;
  if (minus)
      {
        minus++ ; /* skip minus */
        strcpy(s, minus) ;
      }
    else
      strcpy(s, s1) ; /* no network */
  strcpy(s1, pm->location) ;
  while (strlen(s1) < 2)
    strcat(s1, "_") ;
  sprintf(fname, "%s_%d%s.%s", s, gConfig.par_create.q660id_priority, s1, pm->channel) ;
  switch (pm->miniseed_action) {
    case MSA_ARC :
    case MSA_FIRST :
    case MSA_INC :
    case MSA_FINAL : /* write or update data */
      if (print_filename)
        printf("%-7s %s\n", MSA_DESC[pm->miniseed_action], fname);
      if (!write_filename)
         return ;
      bad = FALSE ;
      if (lib_file_open (&gConfig.fowner, fname, LFO_OPEN | LFO_WRITE, &afile) ) /* should work except for first time */
          bad = lib_file_open (&gConfig.fowner, fname, LFO_CREATE | LFO_WRITE, &afile) ; /* might be first time */
      if (bad)
          {
            printf ("Could not open or create %s\n", fname) ;
            return ;
          }
      lib_file_size (&gConfig.fowner, afile, &size) ;
      if (pm->miniseed_action >= MSA_INC)
          lib_file_seek (&gConfig.fowner, afile, size - ARC_RECORD_SIZE) ; /* updating last record */
        else
          lib_file_seek (&gConfig.fowner, afile, size) ; /* to end */
      lib_file_write (&gConfig.fowner, afile, pm->data_address, ARC_RECORD_SIZE) ;
      lib_file_close (&gConfig.fowner, afile) ;
      break ;
    case MSA_GETARC : /* return the last record */
      if (! lib_file_open (&gConfig.fowner, fname, LFO_OPEN | LFO_READ, &afile))
          {
            lib_file_size (&gConfig.fowner, afile, &size) ;
            if (size >= ARC_RECORD_SIZE)
                {
                  lib_file_seek (&gConfig.fowner, afile, size - ARC_RECORD_SIZE) ;
                  lib_file_read (&gConfig.fowner, afile, pm->data_address, ARC_RECORD_SIZE) ;
                  pm->miniseed_action = MSA_RETARC ; /* returning last written */
                }
            lib_file_close (&gConfig.fowner, afile) ;
          }
      break ;
    default :
      break ;
  }
}

void q660_low_latency_callback (pointer p)
{
  size_t size;
  TracePacket tp;
  tlowlat_call *data;
  double timestamp;
  if (!gConfig.send_trace || !lldatacontext)
    return;
  data = p;
  timestamp = data->timestamp;
  if (timestamp < OCT_1_2016 || timestamp > MAX_DATE) {
    return; // impossible time
  }
  tp.trh2.pinno = 0;
  tp.trh2.version[0] = TRACE2_VERSION0;
  tp.trh2.version[1] = TRACE2_VERSION1;
  strcpy(tp.trh2.datatype,TH2_DATATYPE);
  memcpy(tp.trh2.quality, "  ", 2); /* FILL THIS IN */
  process_rate(&tp, data->rate, data->sample_count, timestamp);
  size = process_station_name(&tp, data->station_name, data->channel, data->location);

  // copy the data
  size = tp.trh2.nsamp * sizeof(U32);
  memcpy(&(tp.msg[sizeof(TRACE2_HEADER)]), data->samples, size);
  size += sizeof(TRACE2_HEADER);

  if (testflag & TEST_PRINTDATA) {
    printf ("LL %-10s %s %s %s %s\n", data->station_name, tp.trh2.sta, tp.trh2.chan, tp.trh2.net, tp.trh2.loc);
  }
  if (!(testflag & TEST_STANDALONE)) {
    // put the data into the ring
    EW_DataLogo.type = TypeTrace;
    if(tport_putmsg(&EW_TraceRegion, &EW_DataLogo, size, (char *)(&tp)) != PUT_OK) {
      logMessage("XXX Error putting TRACE packet into ring\n");
    }
  }
}

void q660_onesec_callback (pointer p)
{
  size_t size;
  TracePacket tp;
  tonesec_call *data = p;
  double timestamp;

  if (!gConfig.send_trace)
    return;
  timestamp = data->timestamp;
  if (timestamp < OCT_1_2016 || timestamp > MAX_DATE) {
    return; // impossible time
  }
  tp.trh2.pinno = 0;
  tp.trh2.version[0] = TRACE2_VERSION0;
  tp.trh2.version[1] = TRACE2_VERSION1;
  strcpy(tp.trh2.datatype,TH2_DATATYPE);
  memcpy(tp.trh2.quality, "  ", 2); /* FILL THIS IN */
  process_rate(&tp, data->rate, data->sample_count, timestamp);
  size = process_station_name(&tp, data->station_name, data->channel, data->location);

  // copy the data
  size = tp.trh2.nsamp * sizeof(U32);
  memcpy(&(tp.msg[sizeof(TRACE2_HEADER)]), data->samples, size);
  size += sizeof(TRACE2_HEADER);

  if (testflag & TEST_PRINTDATA) {
    printf ("1S %-10s %s %s %s %s\n", data->station_name, tp.trh2.sta, tp.trh2.chan, tp.trh2.net, tp.trh2.loc);
  }
  if (!(testflag & TEST_STANDALONE)) {
    // put the data into the ring
    EW_DataLogo.type = TypeTrace;
    if(tport_putmsg(&EW_TraceRegion, &EW_DataLogo, size, (char *)(&tp)) != PUT_OK) {
      logMessage("XXX Error putting packet into ring\n");
    }
  }
}
